#include "main.h"
#include "packet.h"
#include "beacon.h"
#include <unistd.h>

// the beacon information is filled in the packet
// send the beacon with an interval of BEACON_INTERVAL
void broadcast_beacon( void *pkt )
{
	int sockfd;
	struct sockaddr_in their_addr; // connector's address information
	struct hostent *he;
	int numbytes;
	int broadcast = 1;
	//char broadcast = '1'; // if that doesn't work, try this
	
	unsigned int broadcast_addr = ((Packet *)pkt)->ip | 0xFF000000;

	// create the socket
	if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}

	// this allows sending packets to the broadcast address
	if (setsockopt(sockfd, SOL_SOCKET, SO_BROADCAST, &broadcast,
				sizeof broadcast) == -1) {
		perror("setsockopt (SO_BROADCAST)");
		exit(1);
	}

	// fill in info about the server (address to send the packet to)
	their_addr.sin_family = AF_INET;     // host byte order
	their_addr.sin_port = htons(BROADCAST_PORT); // short, network byte order
	their_addr.sin_addr.s_addr = broadcast_addr;
	memset(their_addr.sin_zero, '\0', sizeof their_addr.sin_zero);

	// send the broadcast packet
	while(1) {
		if ((numbytes=sendto(sockfd, pkt, sizeof(Packet), 0,
						(struct sockaddr *)&their_addr, sizeof their_addr)) == -1) {
			perror("sendto");
			exit(1);
		}
		printf("broadcast(): sent %d bytes to %s\n", numbytes, inet_ntoa(their_addr.sin_addr));
		sleep(BEACON_INTERVAL);	
		printf("broadcast(): thread awake\n");
	}
	close(sockfd);
}

int test_main()
{
	Packet p;

	// initialize fields
	p.id = 1;
	strcpy(p.mac, "ABCDEFGHIJKL");
	p.ip = inet_addr("128.208.4.14");
	p.port = htonl((long)2190);

	broadcast_beacon(&p);
}
