/* Includes */
#include <hildon-widgets/hildon-program.h>
#include <hildon-widgets/hildon-banner.h>

#include <gtk/gtkmain.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtk.h>
#include <pthread.h>
#include <stdio.h>
#include <time.h>

#include "main.h"
#include "beacon.h"
#include "packet.h"
#include "tcp.h"
#include "gui-main.h"

pthread_t beacon_sender, beacon_receiver, tcp_server;
list **head;
int current_ip, current_port;

int main(int argc, char *argv[])
{
	if(argc != 3)
	{
		fprintf(stderr, "Usage:\n\t %s <MAC-ADDRESS> <IP-Address>\n", argv[0]);
		exit(-1);
	}

	printf("Starting !\n");
	/* Initialize the GTK. */
	gtk_init(&argc, &argv);

	/* Create the hildon program and setup the title */
	program = HILDON_PROGRAM(hildon_program_get_instance());
	g_set_application_name("Layout Example");

	/* Create HildonWindow and set it to HildonProgram */
	window = HILDON_WINDOW(hildon_window_new());
	hildon_program_add_window(program, window);
	
	/* Create the hBoxes and vBoxes to help with the layout */
	main_box = gtk_hbox_new(FALSE, 5);
	left_box = gtk_vbox_new(FALSE, 2);
	right_box = gtk_vbox_new(FALSE, 2);
	button_box = gtk_vbox_new(FALSE, 2);

  	text_area = gtk_text_view_new();
	gtk_text_view_set_editable(GTK_TEXT_VIEW(text_area), FALSE);
	text_frame = gtk_frame_new(NULL);
	text_scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(text_scroll), 
																 GTK_POLICY_AUTOMATIC,
																 GTK_POLICY_AUTOMATIC);
	text_buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_area));
	
	list_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
	list_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(list_store));
	list_frame = gtk_frame_new(NULL);
	list_scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(list_scroll), 
																 GTK_POLICY_AUTOMATIC,
																 GTK_POLICY_AUTOMATIC);
	/* Set the rendering stuff for the TreeView */
	str_renderer = gtk_cell_renderer_text_new();
	str_column = gtk_tree_view_column_new_with_attributes("String",
																												str_renderer,
																												"text",
																												0, NULL);

	gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), str_column);

	int_renderer = gtk_cell_renderer_text_new();
	int_column = gtk_tree_view_column_new_with_attributes("String",
																												int_renderer,
																												"text",
																												1, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), int_column);
	
	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(list_view));
	gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);

	/*
	gtk_tree_view_set_grid_lines(GTK_TREE_VIEW(list_view),GTK_TREE_VIEW_GRID_LINES_VERTICAL);
	*/

	input_area = gtk_entry_new();
	input_frame = gtk_frame_new(NULL);

	gtk_widget_set_size_request(list_view, 400, 300);
	gtk_widget_set_size_request(input_area, 400, 40);

	/* Add the frames to the respective elements */
	gtk_container_add(GTK_CONTAINER(list_scroll), list_view);
	gtk_container_add(GTK_CONTAINER(list_frame), list_scroll);
	
	gtk_container_add(GTK_CONTAINER(input_frame), input_area);
	
	gtk_container_add(GTK_CONTAINER(text_scroll), text_area);
	gtk_container_add(GTK_CONTAINER(text_frame), text_scroll);
	
	/* Add main box to window */
	gtk_container_add(GTK_CONTAINER(window), main_box);
	
	/* Add the buttons / labels to the vbox */
  button1 = gtk_button_new_with_label("Scan");
	gtk_box_pack_start(GTK_BOX(button_box), button1, FALSE, FALSE, 0);
  
	button2 = gtk_button_new_with_label("Get Directory");
	gtk_box_pack_start(GTK_BOX(button_box), button2, FALSE, FALSE, 0);

  button3 = gtk_button_new_with_label("Get File");
	gtk_box_pack_start(GTK_BOX(button_box), button3, FALSE, FALSE, 0);
	
  button4 = gtk_button_new_with_label("Get Selection");
	gtk_box_pack_start(GTK_BOX(button_box), button4, FALSE, FALSE, 0);
	
	/* Add the button box and text area to the right pane */
	gtk_box_pack_start(GTK_BOX(right_box), button_box, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(right_box), text_frame, TRUE, TRUE, 0);

	/* Add the list display to the left pane */
	gtk_box_pack_start(GTK_BOX(left_box), list_frame, TRUE, TRUE, 0);
	gtk_box_pack_end(GTK_BOX(left_box),  input_frame, FALSE, FALSE, 0);

	/* Add stuff to the main box */
	gtk_box_pack_start(GTK_BOX(main_box), left_box, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(main_box), right_box, TRUE, TRUE, 0);

	/* Begin the main application */
	gtk_widget_show_all(GTK_WIDGET(window));

	/* Connect signal to X in the upper corner */
	g_signal_connect(G_OBJECT(window), "delete_event",
			G_CALLBACK(gtk_main_quit), NULL);
	
	g_signal_connect(G_OBJECT(button1), "clicked",
			G_CALLBACK(button_cb), (gpointer) 1);

	g_signal_connect(G_OBJECT(button2), "clicked",
			G_CALLBACK(button_cb), (gpointer) 2);

	g_signal_connect(G_OBJECT(button3), "clicked",
			G_CALLBACK(button_cb), (gpointer) 3);

	g_signal_connect(G_OBJECT(button4), "clicked",
			G_CALLBACK(button_cb), (gpointer) 4);


	/* ---------------- Start of Application Code ---------------------------*/
	list *temp;
	head = &temp;
	*head = NULL;

	Packet bcast_pkt;
	bcast_pkt.id = htonl((long)1);
	strcpy(bcast_pkt.mac, argv[1]);
	bcast_pkt.ip = inet_addr(argv[2]); /* no need to do htonl ... inet_addr returns value in network byte order */
	bcast_pkt.port = htonl((long)TCP_PORT);

	printf("Starting threads!\n");
	pthread_create(&beacon_receiver, NULL, (void *)start_listener, (void *)head); 
	pthread_create(&beacon_sender, NULL, (void *)broadcast_beacon, (void *)&bcast_pkt);
	pthread_create(&tcp_server, NULL, (void *)start_tcp_server, (void *)NULL);
	
	printf("Starting gtk_main!\n");
	gtk_main();
	/* Exit */
	return 0;
}

/******************* Helper fucntions for manipulating the GUI ********************/

/** 
 * Clear the Debug window
 */
void debug_window_clear()
{
	gtk_text_buffer_set_text(text_buf, "", -1);
}

/**
 * Append a string to the debug window.
 */
void debug_window_append(const gchar *text) 
{
	gtk_text_buffer_insert_at_cursor(text_buf, text, -1);
	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(text_area), 
															 gtk_text_buffer_get_insert(text_buf),
															 0, FALSE, 0,	0);
}

/**
 * The function called when any button is clicked.
 * Use the value of i to figure out which button
 * was pressed, and do the required action
 */
void button_cb(GtkWidget *widget, gpointer id)
{
	int i = (int)id;
	char buf[257];
	int intval;
	time_t current_time;
  gchar *str;

	if(i==1) /* Scan */
	{
		current_time = time(0);
		/* Read from the list */
		list_window_clear();
		list *current = *head;
		while(current != NULL)
		{
			list_window_append(current->mac, (current_time-current->timestamp));
			current = current->next;
		}
	}
	else if(i==2) /* Get Dir Listing */
	{
		list_window_get_current_selection(&str, &intval);
		sprintf(buf, "Selected=  %s : %d\n", str, intval);
		debug_window_append(buf);
		if(str==NULL)
		{
			show_banner("Invalid Selection");
			return;
		}
		
		list *neighbor = findNeighbor(head, str);
		if(neighbor==NULL)
		{
			show_banner("Neighbor Missing");
			return;
		}

		int ip = neighbor->ip;
		int port = neighbor->port;
		current_ip = ip;
		current_port = port;
		int sockfd = connect_to_host(ip, port);
		if(sockfd == -1) 
		{
			show_banner("Couldn't connect to neighbor");
			return;
		}
		if(request_file_list(sockfd) == -1)
		{
			show_banner("Couldn't obtain file_list");
			return;
		}
		if(receive_and_write_file_list(sockfd, "tmp.filelist")==-1)
		{
			show_banner("Couldn't obtain file_list");
			return;
		}
		close(sockfd);
		/* now read the list of files and add to the GUI */
		FILE *fp = fopen("tmp.filelist", "r");
		if(!fp)
		{
			show_banner("Error reading from file");
			return;
		}
		char line[257];
		list_window_clear();
		int i = 1;
		while(fgets(line, 256, fp) != NULL) 
		{
			line[strlen(line)-1] = '\0'; /* remove the newline */
			list_window_append(line, i++);
		}
		
	}
	else if(i==3)
	{
		list_window_get_current_selection(&str, &intval);
		/* we still don't keep track of if we're seeing the list
		// of machines, or the list of files!
		*/
		if(str==NULL)
		{
			show_banner("Invalid Selection");
			return;
		}
		int sockfd = connect_to_host(current_ip, current_port);
		if(sockfd == -1) 
		{
			struct in_addr a;
			a.s_addr = current_ip;
			sprintf(buf, "Couldn't connect to neighbor %s:%d", inet_ntoa(a), ntohl(current_port));
			show_banner(buf);
			return;
		}
		if(request_file(sockfd, str)==-1)
		{
			sprintf(buf, "Request failed for file %s", str);
			show_banner(buf);
			close(sockfd);
			return;
		}
		int retval = 0;
		if((retval=receive_file(sockfd, str))<0)
		{
			if(retval==-2)
				sprintf(buf, "File not available at server%s", str);
			else
				sprintf(buf, "Error while receiving file %s", str);

			show_banner(buf);
			close(sockfd);
			return;
		}
	}
	else if(i==4)
	{
		list_window_get_current_selection(&str, &intval);
		sprintf(buf, "Selected=  %s : %d\n", str, intval);
		debug_window_append(buf);
	}
}

/**
* Show a popup banner
*/
void show_banner(gchar* text)
{
		hildon_banner_show_information(GTK_WIDGET(window),NULL,text);
}

/* Set progress bar */
void set_progress(double fraction)
{
	hildon_banner_set_fraction(HILDON_BANNER(banner), fraction);
}

void start_file_transfer(gchar* filename)
{
	gtk_widget_set_sensitive(button2, FALSE);
	char buf[512];
	sprintf(buf, "Receiving file : %s", filename);
	banner = hildon_banner_show_progress(GTK_WIDGET(window), NULL, buf);
}

void end_file_transfer()
{
	gtk_widget_destroy(banner);
	gtk_widget_set_sensitive(button2, TRUE);
}

/**
 * Use this function to get the text from the Input Field.
 */
const gchar* get_input_text()
{
	return gtk_entry_get_text(GTK_ENTRY(input_area));
}

/**
 * Clear the List window (the big main window on the left)
 */
void list_window_clear()
{
	gtk_list_store_clear(list_store);
}

/**
 * Add an entry to the list window. An entry in the list 
 * window has a string and an int as the two columns which are
 * displayed.
 */
void list_window_append(const gchar *text, gint int_val)
{
	/* GtkTreeIter is a pointer to a row in our list display */
	GtkTreeIter iter;
	
	/* Get a pointer to the last row, since we are appending.
	 * Note that we are passing iter by reference, so the pointer
	 * will be stored in iter when the function returns
	 */
	gtk_list_store_append(list_store, &iter);
	
	/* Set the value of the row pointed to by iter */
	gtk_list_store_set(list_store, &iter, 0, text, 1, int_val, -1);
}

/** 
 * Get the value of the current selection. Note that this function is
 * tailored to our list representation (string, int). Arguments are
 * a pointer to pointer to string, and pointer to int
 */
void list_window_get_current_selection(gchar **str, int *int_val)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	if(gtk_tree_selection_get_selected(select, &model, &iter))
	{
		/* Get the value stored in the row pointed to by iter.
		 * Store the value of the first column in str
		 * Store the value of the second column in *int_val
		 */
		gtk_tree_model_get(model, &iter, 0, str, 1, int_val, -1);
	}
}


