#include "tcp.h"
#include "main.h"
#include <pthread.h>

// ip and port are in 4-byte network byte order
int connect_to_host(int ip, int port)
{
	int sockfd, numbytes;  
	char buf[MAXBUFLEN];
	struct hostent *he;
	struct sockaddr_in their_addr; // connector's address information 
	int hport = ntohl(port);

	// create a tcp socket
	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
	    perror("socket");
			return -1;
	}
	
	// fill in server info
	their_addr.sin_family = AF_INET;    // host byte order 
	their_addr.sin_port = htons(hport);  // short, network byte order 
	their_addr.sin_addr.s_addr = ip;
	memset(their_addr.sin_zero, '\0', sizeof their_addr.sin_zero);

	printf("IP = %d = %s\n", ip, inet_ntoa(their_addr.sin_addr));
	// make a tcp connection to the server
	if (connect(sockfd, (struct sockaddr *)&their_addr, sizeof their_addr) == -1) {
	    perror("connect failed:");
			return -1;
	}
	else
		printf("Connected on socket %d\n", sockfd);
	
	return sockfd;
}

// receive file list .. read till connection is closed
int receive_file_list(int sockfd)
{
	char buf[MAXBUFLEN];
	int numbytes;

	while(1)
	{
		if ((numbytes=recv(sockfd, buf, MAXBUFLEN-1, 0)) == -1) {
			perror("recv");
			return -1;
		}
		if(numbytes==0)
			break;
		buf[numbytes] = '\0';

		printf("Received: %s",buf);
		printf("Received: %d bytes\n", numbytes);
	}
}

// receive file list .. read till connection is closed
// and dump to file
int receive_and_write_file_list(int sockfd, char *fname)
{
	char buf[MAXBUFLEN];
	int numbytes;
	FILE *fp = fopen(fname, "w");
	if(!fp)
		return -1;

	while(1)
	{
		if ((numbytes=recv(sockfd, buf, MAXBUFLEN-1, 0)) == -1) {
			perror("recv");
			return -1;
		}
		if(numbytes==0)
			break;
		buf[numbytes] = '\0';
		fprintf(fp, "%s", buf);
		printf("Received: %s",buf);
		printf("Received: %d bytes\n", numbytes);
	}
	fclose(fp);
}

int receive_file(int sockfd, char *fname)
{
	char filename[512];
	FILE *fp;
	strcpy(filename, "received/");
	strcat(filename, fname);
	int numbytes, totalbytes=0;
	int fileSize = 0;
	pthread_t file_thread;
	char buf[5];
	
	// receive file
	if(!(fp = fopen(filename, "w")))
	{
		perror("Couldn't open file for writing\n");
		return -1;
	}
	
	numbytes=recv(sockfd, buf, 4, 0); // get the size
	if(numbytes == -1)
	{
			perror("numbytes = -1 in recv file Size");
			fclose(fp);
			return -2;
	}
	
	fileSize = ntohl(((int *)buf)[0]);
	printf("fileSize = %d\n", fileSize);
	fInfo.sockfd = sockfd;
	fInfo.fp = fp;
	fInfo.file_size = fileSize;
	
	start_file_transfer(fname);
	pthread_create(&file_thread, NULL, (void *)get_file, (void *)&fInfo);
	return 0;
}

int request_file_list(int sockfd)
{
	char request[2];
	request[0] = (char)2; // size of the request
	request[1] = (char)1; // opcode for directory listing
	
	return tcp_sendall(sockfd, request, 2);
}

int request_file(int sockfd, char *fname)
{
	printf("Requesting file: %s\n", fname);
	int length = strlen(fname) + 1 + 1; // for length, opcode
	char *request = (char *)malloc(sizeof(char)*length);
	request[0] = (char)length;
	request[1] = (char)2; // opcode

	strcpy(request+2, fname);

	return tcp_sendall(sockfd, request, length);
}

void* get_file(void *ptr)
{
	struct FileInfo *fI = (struct FileInfo*)ptr;
	int sockfd = fI->sockfd;
	char buf[FILE_BUF_SIZE];
	FILE *fp = fI->fp;
	int file_size = fI->file_size;
	int numbytes=0;
	int totalbytes=0;
	int i=1;
	printf("SOCKFD = %d\n", sockfd);	
	printf("SOCKFD = %d\n", sockfd);
	while(1)
	{
		if ((numbytes=recv(sockfd, buf, FILE_BUF_SIZE, 0)) == -1) {
			perror("numbytes = -1 in recv");
			fclose(fp);
			close(sockfd);
			end_file_transfer();
			return;
		}

		printf("Received: %d bytes\n", numbytes);
		totalbytes += numbytes;
		// update the progress bar only every 10 iterations (to avoid overloading the GUI)
		if(i%10==0)
			set_progress((double)totalbytes/(double)file_size);
		i++;
		if(numbytes==0)
			break;
		//printf("Received: %s",buf);

		if(fwrite(buf, sizeof(char), numbytes, fp) <= 0)
			perror("Error writing");
	}
	fclose(fp);
	close(sockfd);
	end_file_transfer();
}

