#include "tcp.h"
#include "main.h"

char file_buf[FILE_BUF_SIZE];
void *start_tcp_server(void *ptr)
{
	int sockfd, new_fd;  // listen on sock_fd, new connection on new_fd
	struct sockaddr_in my_addr;	// my address information
	struct sockaddr_in their_addr; // connector's address information
	socklen_t sin_size;
	char file_name[MAXBUFLEN];
	char buf[MAXBUFLEN];
	int numbytes;

	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}
	
	// specify that the address is to be reused: useful if the server is stopped 
	// and immediately restarted before the port has been reclaimed by the kernel
	int yes = 1;
	if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1) {
		perror("setsockopt");
		exit(1);
	}

	my_addr.sin_family = AF_INET;		 // host byte order
	my_addr.sin_port = htons(TCP_PORT);	 // short, network byte order
	my_addr.sin_addr.s_addr = INADDR_ANY; // automatically fill with my IP
	memset(my_addr.sin_zero, '\0', sizeof my_addr.sin_zero);

	if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof my_addr) == -1) {
		perror("bind");
		exit(1);
	}

	if (listen(sockfd, BACKLOG) == -1) {
		perror("listen");
		exit(1);
	}

	while(1) {  // main accept() loop
		sin_size = sizeof their_addr;
		if ((new_fd = accept(sockfd, (struct sockaddr *)&their_addr, &sin_size)) == -1) {
			perror("accept");
			continue;
		}
		printf("tcp_server: got connection from %s\n",inet_ntoa(their_addr.sin_addr));
		
		// now get the request
		if ((numbytes=recv(new_fd, buf, MAXBUFLEN-1, 0)) == -1) {
			perror("recv");
			exit(1);
		}
		if(buf[1] == (char)1) // request for dir listing
			send_dir(new_fd, SHARE);
		else if(buf[1] == (char)2) // request for file
		{
			memcpy(file_name, buf+2, buf[0]-2);
			file_name[buf[0]-2] = '\0';
			printf("Sending file: %s\n", file_name);
			send_file(new_fd, file_name);
		}
		else
			printf("Error: Malformed request %d %d\n", buf[0], buf[1]);
		close(new_fd);
	}

	return 0;
}

int send_dir(int sockfd, char *dir)
{
	DIR             *dip;
	struct dirent   *dit;

	int             i = 0, numbytes;
	char						dname[256]; // max length of a filename

	/* DIR *opendir(const char *name);
	 *
	 * Open a directory stream to 'dir' and make sure
	 * it's a readable and valid (directory) */
	if ((dip = opendir(dir)) == NULL)
	{
		perror("opendir");
		return 0;
	}

	printf("Directory stream is now open\n");

	/*  struct dirent *readdir(DIR *dir);
	 *
	 * Read in the files from argv[1] and print */
	while ((dit = readdir(dip)) != NULL)
	{
		i++;
		strcpy(dname, dit->d_name);
		if(dname[0] == '.')
			continue;
		dname[strlen(dit->d_name)] = '\n';
		if ((numbytes=send(sockfd, dname, strlen(dit->d_name)+1, 0)) == -1)
			perror("send");
		printf("\n%s , %d bytes", dit->d_name, numbytes);
	}

	printf("\n\nreaddir() found a total of %i files\n", i);

	/* int closedir(DIR *dir);
	 *
	 * Close the stream to argv[1]. And check for errors. */
	if (closedir(dip) == -1)
	{
		perror("closedir");
		return 0;
	}

	printf("\nDirectory stream is now closed\n");
	return 1;
}

int send_file(int sockfd, char *filename)
{
	FILE *fp;
	char fname[512];
	int bytesRead=0, totalBytes=0, ret_val, fileSize=0;
	strcpy(fname, SHARE);
	strcat(fname, "/");
	strcat(fname, filename);
	fp = fopen(fname, "r");
	if(!fp) 
	{
		printf("Error reading file %s\n", fname);
		((int *)file_buf)[0] = htonl((long)-1); // remember to send the file length in network-byte order as well
		if (tcp_sendall(sockfd, file_buf, 4) == -1)
			perror("send");
		return -1;
	}
	printf("Reading file %d: %s : %d\n", fp, fname, strlen(fname));
	fseek(fp, 0L, SEEK_END);
	fileSize = ftell(fp);
	fseek(fp, 0L, SEEK_SET);

	printf("Filesize = %d\n", fileSize);
	((int *)file_buf)[0] = htonl((int)fileSize);
	
	if (tcp_sendall(sockfd, file_buf, 4) == -1)
		perror("send");
	
	
while((bytesRead=fread(file_buf, 1, FILE_BUF_SIZE, fp)) > 0)
	{
		totalBytes += bytesRead;
		printf("Read %d bytes, total so far %d bytes\n", bytesRead, totalBytes);
		if ((ret_val=tcp_sendall(sockfd, file_buf, bytesRead)) == -1)
			perror("send");
		//printf("ret_val = %d\n", ret_val);
	}
	if(ferror(fp))
	{
		printf("Error while reading file %d\n", fp);
		return -1;
	}

	if(feof(fp))
		printf("Read upto EOF %d\n", fp);
	
}
