/**
Broadcast receiver: runs in a thread, and listens for beacons.
Updates the neighbor-list when a beacon is received
*/

#include "beacon.h"
#include "packet.h"
#include "utils/list.h"
#include "main.h"
#include "utils/network.h"

void start_listener(void *ptr)
{
	list **neighbor_list = (list **)ptr;
	int sockfd;
	struct sockaddr_in my_addr;    // my address information
	struct sockaddr_in their_addr; // connector's address information
	socklen_t addr_len;
	
	int numbytes;
	char buf[RCV_BUF_LEN];
	char print_buf[256];
	printf("Started listener!!\n");
	debug_window_append("Started listener\n");
	// create a socket
	if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		perror("socket");
		exit(1);
	}

	my_addr.sin_family = AF_INET;         // host byte order
	my_addr.sin_port = htons(BROADCAST_PORT);     // short, network byte order
	my_addr.sin_addr.s_addr = INADDR_ANY; // automatically fill with my IP
	memset(my_addr.sin_zero, '\0', sizeof my_addr.sin_zero);
	
	// bind the socket to the port
	if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof my_addr) == -1) {
		perror("bind");
		exit(1);
	}

	addr_len = sizeof their_addr;

	// recvfrom waits for a packet and returns the number of bytes read ...
	// typically this will be in a while loop to ensure that the whole message
	// is read
	while(1) // keep listening!
	{
		if ((numbytes = recvall(sockfd, (char*)&buf, RCV_BUF_LEN-1,
						(struct sockaddr_in *)&their_addr)) == -1) {
			perror("recvall");
			exit(1);
		}

		printf("got packet from %s\n",inet_ntoa(their_addr.sin_addr));
		printf("packet is %d bytes long\n",numbytes);

		addNeighbor(neighbor_list, (Packet *)buf);
//		printList(neighbor_list);
	}
	close(sockfd);

}

/*
int test_main()
{
	list *temp;
	list **neighbor_list = &temp;
	*neighbor_list = NULL;

	start_listener(neighbor_list);
}
*/
