#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "list.h"

/*
// add neighbor to the list
// replace previous entry for this mac if any
// update the timestamp
*/
void addNeighbor(list **head, Packet *pkt)
{
	list *current = *head;
	char pkt_mac[12];
	memcpy(pkt_mac, pkt->mac, 12);
	list *new_item = findNeighbor(head, pkt->mac);
	
	if(new_item != NULL) /* neighbor exists */
	{
		/* update the ip and port, in case it has changed */
		strcpy(new_item->mac, pkt->mac);
		new_item->ip = pkt->ip;
		new_item->port = pkt->port;
		new_item->timestamp = time(0);
		return;
	}
	
	/* element doesn't exist */
	new_item = (list*)(malloc(sizeof(list)));
	strcpy(new_item->mac, pkt->mac);
	new_item->ip = pkt->ip;
	new_item->port = pkt->port;
	new_item->timestamp = time(0);

	new_item->next = current;
	*head = new_item;
}

/*
// returns a pointer to the list element
// contianing info about this mac
// or NULL
*/
list *findNeighbor(list **head, char *mac)
{
	list *current = *head;

	while(current != NULL)
	{
		if(memcmp(current->mac, mac, 12)==0)
			return current;

		current = current->next;
	}

	return NULL; 
}

void printList(list **head)
{
	list *current = *head;
	printf("List: \n");
	while( current != NULL )
	{
		printNeighbor(current);
		current = current->next;
	}
	printf("NULL\n");
}

void printNeighbor(list *current)
{
	struct in_addr ina;
	ina.s_addr = current->ip;
	printf("%s: IP=%s, Port=%d, Timestamp=%ld\n", current->mac, inet_ntoa(ina), ntohl(current->port), current->timestamp);
}

/*
int test_main()
{
	list *temp;
	list **head = &temp ;
	*head = NULL;
//	strcpy(item->mac, "First!");
//	item->next = NULL;

	printList(head);
	addItem(head, "Next2");
	addItem(head, "Next3");
	printList(head);
	addItem(head, "Next4");
	addItem(head, "Next5");
	printList(head);
}
*/
