#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>
#include <stdio.h>

#include "network.h"

int recvall(int handle, char * buf, int sz, struct sockaddr_in *addr)
{
	int rv;
	socklen_t addr_len = sizeof(*addr);
	DEBUG(5, "Enter recvall %d\n", handle);
	while(1)
	{
		rv = recvfrom(handle, buf, sz, 0, (struct sockaddr*)addr, &addr_len);
		if (rv < 0)
		{
			if (errno == EINTR)
			    continue;
			perror("Error encountered in recvall: ");
			DEBUG(5, "Exit recvall %d -1\n", handle);
			return -1;
		}
		break;
	}
	DEBUG(5, "Exit recvall %d %d\n", handle, rv);
	return rv;
}

int sendall(int handle, char * buf, int sz, struct sockaddr_in *addr)
{
	int rv;
	DEBUG(5, "Enter sendall %d %d\n", handle, sz);
	while(sz>0)
	{
		if ((rv = sendto(handle, buf, sz, 0, (struct sockaddr*)addr, sizeof(*addr)))<0)
		{
			int err_code;
			if (errno == EINTR)
				continue;
			err_code = errno;
			perror("Error encountered in sendall: ");
			DEBUG(5, "Exit sendall %d %d\n", handle, err_code);
			return -1;
		}
		DEBUG(5, "sendto returned %d\n", rv);
		sz-=rv;
		buf+=rv;
	}
	DEBUG(5, "Exit sendall %d %d\n", handle, sz);
	return 1;
}

/* return -1 on error, +1 on success */
int tcp_sendall(int handle, char * buf, int sz)
{
	int rv;
	DEBUG(5, "Enter sendall %d %d\n", handle, sz);
	while(sz>0)
	{
		if ((rv = send(handle, buf, sz, 0))<0)
		{
			int err_code;
			if (errno == EINTR)
				continue;
			err_code = errno;
			perror("Error encountered in sendall: ");
			DEBUG(5, "Exit sendall %d %d\n", handle, err_code);
			return -1;
		}
		DEBUG(5, "sendto returned %d\n", rv);
		sz-=rv;
		buf+=rv;
	}
	DEBUG(5, "Exit sendall %d %d\n", handle, sz);
	return 1;
}
